package com.instaply.sdk.proxy.controller;

import static java.util.stream.Collectors.toList;

import com.instaply.sdk.proxy.dto.ExceptionDto;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

@ControllerAdvice
public class ExceptionHandlerControllerAdvice extends ResponseEntityExceptionHandler {

  private static final Logger logger = LoggerFactory.getLogger(ExceptionHandlerControllerAdvice.class);

  private static final String INVALID_JSON_CODE = "40000";
  private static final String INVALID_ARG_CODE = "40001";
  private static final String UNHANDLED_EXCEPTION_CODE = "50000";

  @ResponseBody
  @ExceptionHandler(Exception.class)
  public ExceptionDto handleException(Exception e, HttpServletResponse response) {
    ExceptionDto body = new ExceptionDto(UNHANDLED_EXCEPTION_CODE, "The server encountered an internal error.");
    response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
    logger.error(body.toString(), e);
    return body;
  }

  @Override
  protected ResponseEntity<Object> handleHttpMessageNotReadable(HttpMessageNotReadableException ex,
                                                                HttpHeaders headers, HttpStatus status,
                                                                WebRequest request) {
    ExceptionDto body = new ExceptionDto(INVALID_JSON_CODE, "Could not read JSON");
    logger.error(body.toString(), ex);
    return handleExceptionInternal(ex, body, headers, status, request);
  }

  @Override
  protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex,
                                                                HttpHeaders headers, HttpStatus status,
                                                                WebRequest request) {
    List<ExceptionDto.ErrorDto> errors = new ArrayList<>();
    errors.addAll(ex.getBindingResult().getGlobalErrors().stream()
        .map(it -> new ExceptionDto.ErrorDto("", it.getDefaultMessage()))
        .collect(toList()));

    errors.addAll(ex.getBindingResult().getFieldErrors().stream()
        .map(it -> new ExceptionDto.ErrorDto(it.getField(), it.getDefaultMessage()))
        .collect(toList()));

    ExceptionDto body = new ExceptionDto(INVALID_ARG_CODE, "Illegal argument", errors);
    logger.error(body.toString());
    for (ExceptionDto.ErrorDto error : errors) {
      logger.error("\t- {}", error.toString());
    }

    return handleExceptionInternal(ex, body, headers, status, request);
  }
}
