package com.instaply.sdk.proxy.validator;

import static java.lang.annotation.ElementType.ANNOTATION_TYPE;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Objects;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.beanutils.BeanUtils;

/**
 * Custom validation constraint that can be used to make sure at least one of the fields is not null.
 * <p>
 * Annotate the class and pass the list of field names as parameter.
 * <pre>
 * {@literal @}NotAllNull("foo", "bar")
 * public class FooBar {
 *   private String foo;
 *   private String bar;
 * }
 * </pre>
 */
@Target({TYPE, ANNOTATION_TYPE})
@Retention(RUNTIME)
@Constraint(validatedBy = NotAllNull.Validator.class)
@Documented
public @interface NotAllNull {
  String[] value();

  String message() default "has properties that cannot be all null";

  Class<?>[] groups() default {};

  Class<? extends Payload>[] payload() default {};

  @Target({TYPE, ANNOTATION_TYPE})
  @Retention(RUNTIME)
  @Documented
  @interface List {
    NotAllNull[] value();
  }

  class Validator implements ConstraintValidator<NotAllNull, Object> {
    private Collection<String> fields;

    @Override
    public void initialize(NotAllNull constraintAnnotation) {
      fields = Arrays.asList(constraintAnnotation.value());
    }

    @Override
    public boolean isValid(final Object value, ConstraintValidatorContext context) {
      return fields.stream()
          .map(field -> {
            try {
              return BeanUtils.getProperty(value, field);
            } catch (IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
              throw new RuntimeException(e);
            }
          })
          .anyMatch(Objects::nonNull);
    }
  }
}
