package com.instaply.sdk.proxy.validator;

import static java.lang.annotation.ElementType.ANNOTATION_TYPE;
import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import com.instaply.sdk.proxy.dto.NotificationDto;
import com.instaply.sdk.proxy.dto.NotificationType;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;
import java.lang.reflect.InvocationTargetException;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.beanutils.BeanUtils;

/**
 * Custom validation constraint that can be used to make sure the {@link NotificationDto} has a valid content relative
 * to its type ({@link NotificationDto#type}).
 * <p>
 * The annotation is only meant to work with the {@link NotificationDto} class. In particular, it expects it to
 * contains a {@link NotificationType} field named `type`
 * </p>
 * <p>
 * Usage: Annotate the class and pass with the field name and the type.
 * </p>
 * <pre>
 * {@literal @}NotNullIfTypeIs("bar", NotificationType.message)
 * public class NotificationDto {
 *   private NotificationType type;
 *   private String bar;
 * }
 * </pre>
 */
@Target({TYPE, ANNOTATION_TYPE})
@Retention(RUNTIME)
@Constraint(validatedBy = NotNullIfTypeIs.Validator.class)
@Documented
public @interface NotNullIfTypeIs {
  String field();

  NotificationType value();

  String message() default "may not be null";

  Class<?>[] groups() default {};

  Class<? extends Payload>[] payload() default {};

  @Target({TYPE, ANNOTATION_TYPE})
  @Retention(RUNTIME)
  @Documented
  @interface List {
    NotNullIfTypeIs[] value();
  }

  class Validator implements ConstraintValidator<NotNullIfTypeIs, NotificationDto> {

    private String field;
    private NotificationType type;

    @Override
    public void initialize(NotNullIfTypeIs constraintAnnotation) {
      field = constraintAnnotation.field();
      type = constraintAnnotation.value();
    }

    @Override
    public boolean isValid(NotificationDto value, ConstraintValidatorContext context) {
      try {
        if (value.getType() != type || BeanUtils.getProperty(value, field) != null) {
          return true;
        }

        context.disableDefaultConstraintViolation();
        context.buildConstraintViolationWithTemplate(context.getDefaultConstraintMessageTemplate())
            .addPropertyNode(field)
            .addConstraintViolation();
        return false;

      } catch (IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
        throw new RuntimeException(e);
      }
    }
  }
}
