package com.instaply.sdk.proxy.controller

import static java.time.format.DateTimeFormatter.ISO_OFFSET_DATE_TIME
import static org.mockito.Matchers.isA
import static org.mockito.Mockito.never
import static org.mockito.Mockito.reset
import static org.mockito.Mockito.times
import static org.mockito.Mockito.verify
import static org.springframework.http.HttpStatus.BAD_REQUEST
import static org.springframework.http.HttpStatus.OK
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.head
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post

import com.instaply.sdk.proxy.IntegrationSpec

import com.notnoop.apns.ApnsService
import com.notnoop.apns.EnhancedApnsNotification

import java.time.Instant
import java.time.ZoneOffset

import groovy.json.JsonBuilder

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.test.web.servlet.MvcResult

import spock.lang.Unroll

class NotificationsControllerSpec extends IntegrationSpec {

  @Autowired
  ApnsService mockApnsService

  def setup() {
    createAndSaveDevice(1L, 'CUST-1', randomToken(), 'apns')
    createAndSaveDevice(2L, 'CUST-1', randomToken(), 'gcm')
    createAndSaveDevice(3L, 'CUST-1', randomToken(), 'unk')
    createAndSaveDevice(4L, 'CUST-2', randomToken(), 'apns')
    createAndSaveDevice(5L, 'CUST-2', randomToken(), 'apns')

    reset(mockApnsService)
  }

  @Unroll
  def "retrieve status (#method)"() {
    when:
    MvcResult result = mockMvc.perform(requestBuilder.accept("application/json")).andReturn()

    then:
    result.response.status == OK.value()

    where:
    method | requestBuilder
    'get'  | get('/notifications')
    'head' | head('/notifications')
  }

  @Unroll
  def "send push notification (#caseName)"() {
    setup:
    def body = validPushNotificationRequest('CUST-1')
    if (removedPath) {
      removeAt(body, removedPath)
    }

    when:
    MvcResult result = mockMvc.perform(post('/notifications')
        .header('ipns-version', '1')
        .accept('application/json')
        .contentType('application/json')
        .content(new JsonBuilder(body).toString()))
        .andReturn()

    then:
    result.response.status == OK.value()
    mockito(verify(mockApnsService, times(1)).push(isA(EnhancedApnsNotification.class)))

    where:
    caseName                   | removedPath
    'full body'                | null
    'organization is optional' | 'message.from.organization'
    'organization picture is optional'  | 'conversation.organization.picture'
    'from user is optional'    | 'message.from.user'
    'from user lastName is optional'    | 'message.from.lastName'
    'from user picture is optional'     | 'message.from.picture'
    'text is optional'                  | 'message.text'
    'attachment is optional'            | 'message.attachment'
    'attachment thumbnails is optional' | 'message.attachment.thumbnails'
  }

  @Unroll
  def "send push notification with #caseName"() {
    given:
    def body = validPushNotificationRequest('CUST-1')
    if (value == null) {
      removeAt(body, paramPath)
    } else {
      putAt(body, paramPath, value)
    }

    when:
    MvcResult result = mockMvc.perform(post('/notifications')
        .accept('application/json')
        .contentType('application/json')
        .header('ipns-version', '1')
        .content(new JsonBuilder(body).toString()))
        .andReturn()

    then:
    result.response.status == BAD_REQUEST.value()
    mockito(verify(mockApnsService, never()).push(isA(EnhancedApnsNotification.class)))

    where:
    caseName                               | paramPath                              | value
    'empty body'                           | ''                                     | null
    'missing type'                         | 'type'                                 | null
    'unknown type'                         | 'type'                                 | 'unknown'
    'missing unreadMessageCount'           | 'unreadMessagesCount'                  | null
    'negative unreadMessageCount'          | 'unreadMessagesCount'                  | -6
    'unreadMessageCount NaN'               | 'unreadMessagesCount'                  | 'NotANumber'
    'missing date'                         | 'date'                                 | null
    'invalid date'                         | 'date'                                 | 'ThisIsNotADate'
    'missing customer'                     | 'customer'                             | null
    'missing customer id'                  | 'customer.id'                          | null
    'blank customer id'                    | 'customer.id'                          | ''
    'missing message'                      | 'message'                              | null
    'missing message from'                 | 'message.from'                         | null
    'missing message from user first name' | 'message.from.user.firstName'          | null
    'blank message from user first name'   | 'message.from.user.firstName'          | ''
    'invalid from user picture url'        | 'message.from.user.picture'            | 'ThisIsNotAUrl'
    'missing organization name'            | 'message.from.organization.name'       | null
    'blank organization name'              | 'message.from.organization.name'       | ''
    'invalid organization picture url'     | 'message.from.organization.picture'    | 'ThisIsNotAUrl'
    'missing attachment name'              | 'message.attachment.name'              | null
    'blank attachment name'                | 'message.attachment.name'              | ''
    'missing attachment content type'      | 'message.attachment.contentType'       | null
    'blank attachment content type'        | 'message.attachment.contentType'       | ''
    'missing attachment thumbnails 50x50'  | 'message.attachment.thumbnails._50x50' | null
    'blank attachment thumbnails url'      | 'message.attachment.thumbnails._50x50' | ''
    'invalid attachment thumbnails url'    | 'message.attachment.thumbnails._50x50' | 'ThisIsNotAUrl'
  }

  def "send push notification with no text nor attachment"() {
    given:
    def body = validPushNotificationRequest('CUST-1')
    removeAt(body, "message.attachment")
    removeAt(body, "message.text")

    when:
    MvcResult result = mockMvc.perform(post('/notifications')
        .accept('application/json')
        .contentType('application/json')
        .header('ipns-version', '1')
        .content(new JsonBuilder(body).toString()))
        .andReturn()

    then:
    result.response.status == BAD_REQUEST.value()
    mockito(verify(mockApnsService, never()).push(isA(EnhancedApnsNotification.class)))
  }


  private def validPushNotificationRequest(def customerId) {
    [
        type               : 'message',
        unreadMessagesCount: 12,
        date               : Instant.now().atOffset(ZoneOffset.UTC).format(ISO_OFFSET_DATE_TIME),
        customer           : [
            id: customerId
        ],
        message            : [
            from      : [
                user        : [
                    firstName: 'Fabien',
                    lastName : 'Degaugue',
                    picture  : 'https://api.instaply.com/users/5/icon'
                ],
                organization: [
                    name   : 'Instaply',
                    picture: 'https://api.instaply.com/files/de3d4c6b-0be4-4bb1-b534-b42272cc3998'
                ]
            ],
            text      : 'hello sdk',
            attachment: [
                name       : 'instaply-logo.png',
                contentType: 'image/png',
                thumbnails : [
                    _50x50: 'https://api.instaply.com/files/de3d4c6b-0be4-4bb1-b534-b42272cc3998?width=50&height=50'
                ]
            ]
        ],
    ]
  }
}