package com.instaply.sdk.proxy.service

import static groovy.json.internal.Charsets.UTF_8
import static org.mockito.Matchers.isA
import static org.mockito.Mockito.never
import static org.mockito.Mockito.reset
import static org.mockito.Mockito.times
import static org.mockito.Mockito.verify

import com.instaply.sdk.proxy.IntegrationSpec
import com.instaply.sdk.proxy.dto.MessageDto
import com.instaply.sdk.proxy.dto.MessageDto.AttachmentDto
import com.instaply.sdk.proxy.dto.MessageDto.AttachmentDto.ThumbnailsDto
import com.instaply.sdk.proxy.dto.MessageDto.FromDto
import com.instaply.sdk.proxy.dto.NotificationDto
import com.instaply.sdk.proxy.dto.NotificationDto.CustomerDto
import com.instaply.sdk.proxy.dto.UserDto

import com.notnoop.apns.ApnsService
import com.notnoop.apns.EnhancedApnsNotification

import java.time.Instant
import java.time.ZoneOffset

import groovy.json.JsonBuilder

import org.mockito.ArgumentCaptor

import org.springframework.beans.factory.annotation.Autowired

import spock.lang.Shared
import spock.lang.Unroll

class PushNotificationServiceSpec extends IntegrationSpec {

  @Autowired
  PushNotificationService pushNotificationService

  @Autowired
  ApnsService mockApnsService

  @Shared
  String expectedApnsPayload

  def setup() {
    def json = new JsonBuilder()
    json {
      aps {
        badge 12
        alert "New message received from Fabien Degaugue: Hello SDK"
      }
      type 'instaply_message'
    }
    expectedApnsPayload = json.toString()

    reset(mockApnsService)
  }

  def 'no push notification sent when request contains no device'() {
    given:
    def request = notification('CUST-3')

    when:
    pushNotificationService.sendPushNotification(request)

    then:
    mockito(verify(mockApnsService, never()).push(isA(EnhancedApnsNotification.class)))
  }

  def "send apns push notification"() {
    given:
    def device = createAndSaveDevice(1L, 'CUST-1', randomToken(), 'apns')

    when:
    pushNotificationService.sendPushNotification(notification('CUST-1'))

    then:
    def argument = ArgumentCaptor.forClass(EnhancedApnsNotification.class);
    mockito(verify(mockApnsService, times(1)).push(argument.capture()))

    argument.value.deviceToken.encodeHex().toString() == device.token
    argument.value.payload == expectedApnsPayload.getBytes(UTF_8)
  }

  @Unroll
  def "send push notification to multiple devices (#caseName)"() {
    given:
    def devices = []
    types.eachWithIndex { type, index ->
      devices << createAndSaveDevice(index, 'CUST-2', randomToken(), type)
    }
    int apnsDevicesCount = devices.count { it.type == 'apns' }

    when:
    pushNotificationService.sendPushNotification(notification('CUST-2'))

    then:
    def argument = ArgumentCaptor.forClass(EnhancedApnsNotification.class);
    mockito(verify(mockApnsService, times(apnsDevicesCount)).push(argument.capture()))

    argument.allValues.each {
      devices.collect { it.token }.contains(it.deviceToken.encodeHex().toString())
      it.payload == expectedApnsPayload.getBytes(UTF_8)
    }

    where:
    caseName    | types
    'apns only' | ['apns', 'apns']
  }

  private NotificationDto notification(def customerId) {
    return new NotificationDto(
        type: 'message',
        unreadMessagesCount: 12,
        date: Instant.now().atOffset(ZoneOffset.UTC),
        customer: new CustomerDto(id: customerId),
        message: new MessageDto(
            from: new FromDto(
                user: new UserDto(
                    firstName: 'Fabien',
                    lastName: 'Degaugue',
                    picture: 'https://api.instaply.com/users/5/icon')
            ),
            text: 'Hello SDK',
            attachment: new AttachmentDto(
                name: 'instaply-logo.png',
                contentType: 'image/png',
                thumbnails: new ThumbnailsDto(
                    _50x50: 'https://api.instaply.com/files/de3d4c6b-0be4-4bb1-b534-b42272cc3998?width=50&height=50'
                )
            )
        )
    )
  }
}
